@@ -4,15 +4,18 @@
 #include "kitmanager.h"
 
 #include "abi.h"
+#include "devicesupport/devicekitaspects.h"
 #include "devicesupport/idevicefactory.h"
 #include "kit.h"
 #include "kitfeatureprovider.h"
-#include "kitinformation.h"
+#include "kitaspect.h"
 #include "projectexplorerconstants.h"
 #include "projectexplorertr.h"
+#include "toolchainkitaspect.h"
 #include "toolchainmanager.h"
 
 #include <coreplugin/icore.h>
+#include <coreplugin/progressmanager/progressmanager.h>
 
 #include <android/androidconstants.h>
 #include <baremetal/baremetalconstants.h>
@@ -26,12 +29,11 @@
 #include <utils/qtcassert.h>
 #include <utils/stringutils.h>
 
-#include <QAction>
+#include <nanotrace/nanotrace.h>
+
 #include <QHash>
-#include <QLabel>
-#include <QPushButton>
-#include <QSettings>
-#include <QStyle>
+
+#include <optional>
 
 using namespace Core;
 using namespace Utils;
@@ -74,40 +76,6 @@ public:
     std::vector<std::unique_ptr<Kit>> m_kitList;
     std::unique_ptr<PersistentSettingsWriter> m_writer;
     QSet<Id> m_irrelevantAspects;
-
-    void addKitAspect(KitAspect *ki)
-    {
-        QTC_ASSERT(!m_aspectList.contains(ki), return);
-        m_aspectList.append(ki);
-        m_aspectListIsSorted = false;
-    }
-
-    void removeKitAspect(KitAspect *ki)
-    {
-        int removed = m_aspectList.removeAll(ki);
-        QTC_CHECK(removed == 1);
-    }
-
-    const QList<KitAspect *> kitAspects()
-    {
-        if (!m_aspectListIsSorted) {
-            Utils::sort(m_aspectList, [](const KitAspect *a, const KitAspect *b) {
-                return a->priority() > b->priority();
-            });
-            m_aspectListIsSorted = true;
-        }
-        return m_aspectList;
-    }
-
-    void setBinaryForKit(const FilePath &fp) { m_binaryForKit = fp; }
-    FilePath binaryForKit() const { return m_binaryForKit; }
-
-private:
-    // Sorted by priority, in descending order...
-    QList<KitAspect *> m_aspectList;
-    // ... if this here is set:
-    bool m_aspectListIsSorted = true;
-
     FilePath m_binaryForKit;
 };
 
@@ -118,40 +86,88 @@ private:
 // --------------------------------------------------------------------------
 
 static Internal::KitManagerPrivate *d = nullptr;
-static KitManager *m_instance = nullptr;
 
 KitManager *KitManager::instance()
 {
-    if (!m_instance)
-        m_instance = new KitManager;
-    return m_instance;
+    static KitManager theManager;
+    return &theManager;
 }
 
 KitManager::KitManager()
 {
     d = new KitManagerPrivate;
-    QTC_CHECK(!m_instance);
-    m_instance = this;
-
-    connect(ICore::instance(), &ICore::saveSettingsRequested, this, &KitManager::saveKits);
-
-    connect(this, &KitManager::kitAdded, this, &KitManager::kitsChanged);
-    connect(this, &KitManager::kitRemoved, this, &KitManager::kitsChanged);
-    connect(this, &KitManager::kitUpdated, this, &KitManager::kitsChanged);
 }
 
 void KitManager::destroy()
 {
     delete d;
     d = nullptr;
-    delete m_instance;
-    m_instance = nullptr;
 }
 
+static bool kitMatchesAbiList(const Kit *kit, const Abis &abis)
+{
+    const QList<Toolchain *> toolchains = ToolchainKitAspect::toolChains(kit);
+    for (const Toolchain * const tc : toolchains) {
+        const Abi tcAbi = tc->targetAbi();
+        for (const Abi &abi : abis) {
+            if (tcAbi == abi)
+                return true;
+        }
+    }
+    return false;
+};
+
+static bool isHostKit(const Kit *kit)
+{
+    const Abi hostAbi = Abi::hostAbi();
+    if (HostOsInfo::isMacHost() && hostAbi.architecture() == Abi::ArmArchitecture) {
+        const Abi x86Abi(Abi::X86Architecture,
+                         hostAbi.os(),
+                         hostAbi.osFlavor(),
+                         hostAbi.binaryFormat(),
+                         hostAbi.wordWidth());
+
+        return kitMatchesAbiList(kit, {hostAbi, x86Abi});
+    }
+    return kitMatchesAbiList(kit, {hostAbi});
+};
+
+static Id deviceTypeForKit(const Kit *kit)
+{
+    if (isHostKit(kit))
+        return Constants::DESKTOP_DEVICE_TYPE;
+    const QList<Toolchain *> toolchains = ToolchainKitAspect::toolChains(kit);
+    for (const Toolchain * const tc : toolchains) {
+        const Abi tcAbi = tc->targetAbi();
+        switch (tcAbi.os()) {
+        case Abi::BareMetalOS:
+            return BareMetal::Constants::BareMetalOsType;
+        case Abi::BsdOS:
+        case Abi::DarwinOS:
+        case Abi::UnixOS:
+            return RemoteLinux::Constants::GenericLinuxOsType;
+        case Abi::LinuxOS:
+            if (tcAbi.osFlavor() == Abi::AndroidLinuxFlavor)
+                return Android::Constants::ANDROID_DEVICE_TYPE;
+            return RemoteLinux::Constants::GenericLinuxOsType;
+        case Abi::QnxOS:
+            return Qnx::Constants::QNX_QNX_OS_TYPE;
+        case Abi::VxWorks:
+            return "VxWorks.Device.Type";
+        default:
+            break;
+        }
+    }
+    return Constants::DESKTOP_DEVICE_TYPE;
+};
+
 void KitManager::restoreKits()
 {
+    NANOTRACE_SCOPE("ProjectExplorer", "KitManager::restoreKits");
     QTC_ASSERT(!d->m_initialized, return );
 
+    connect(ICore::instance(), &ICore::saveSettingsRequested, &KitManager::saveKits);
+
     std::vector<std::unique_ptr<Kit>> resultList;
 
     // read all kits from user file
@@ -196,11 +212,12 @@ void KitManager::restoreKits()
                 Kit *ptr = i->get();
 
                 // Overwrite settings that the SDK sets to those values:
-                for (const KitAspect *aspect : KitManager::kitAspects()) {
+                for (const KitAspectFactory *factory : KitManager::kitAspectFactories()) {
+                    const Id id = factory->id();
                     // Copy sticky settings over:
-                    ptr->setSticky(aspect->id(), toStore->isSticky(aspect->id()));
-                    if (ptr->isSticky(aspect->id()))
-                        ptr->setValue(aspect->id(), toStore->value(aspect->id()));
+                    ptr->setSticky(id, toStore->isSticky(id));
+                    if (ptr->isSticky(id))
+                        ptr->setValue(id, toStore->value(id));
                 }
                 toStore = std::move(*i);
                 kitsToCheck.erase(i);
@@ -224,129 +241,44 @@ void KitManager::restoreKits()
         }
     }
 
-    static const auto kitMatchesAbiList = [](const Kit *kit, const Abis &abis) {
-        const QList<ToolChain *> toolchains = ToolChainKitAspect::toolChains(kit);
-        for (const ToolChain * const tc : toolchains) {
-            const Abi tcAbi = tc->targetAbi();
-            for (const Abi &abi : abis) {
-                if (tcAbi.os() == abi.os() && tcAbi.architecture() == abi.architecture()
-                        && (tcAbi.os() != Abi::LinuxOS || tcAbi.osFlavor() == abi.osFlavor())) {
-                    return true;
-                }
-            }
-        }
-        return false;
-    };
-
-    const Abis abisOfBinary = d->binaryForKit().isEmpty()
-            ? Abis() : Abi::abisOfBinary(d->binaryForKit());
-    const auto kitMatchesAbiOfBinary = [&abisOfBinary](const Kit *kit) {
-        return kitMatchesAbiList(kit, abisOfBinary);
-    };
+    const Abis abisOfBinary = d->m_binaryForKit.isEmpty()
+            ? Abis() : Abi::abisOfBinary(d->m_binaryForKit);
     const bool haveKitForBinary = abisOfBinary.isEmpty()
-            || contains(resultList, [&kitMatchesAbiOfBinary](const std::unique_ptr<Kit> &kit) {
-        return kitMatchesAbiOfBinary(kit.get());
+            || contains(resultList, [&abisOfBinary](const std::unique_ptr<Kit> &kit) {
+        return kitMatchesAbiList(kit.get(), abisOfBinary);
     });
     Kit *kitForBinary = nullptr;
 
+    QList<Kit *> hostKits;
     if (resultList.empty() || !haveKitForBinary) {
         // No kits exist yet, so let's try to autoconfigure some from the toolchains we know.
-        QHash<Abi, QHash<Utils::Id, ToolChain *>> uniqueToolchains;
+        QHash<Abi, QHash<LanguageCategory, std::optional<ToolchainBundle>>> uniqueToolchains;
 
-        // On Linux systems, we usually detect a plethora of same-ish toolchains. The following
-        // algorithm gives precedence to icecc and ccache and otherwise simply chooses the one with
-        // the shortest path. This should also take care of ensuring matching C/C++ pairs.
-        // TODO: This should not need to be done here. Instead, it should be a convenience
-        // operation on some lower level, e.g. in the toolchain class(es).
-        // Also, we shouldn't detect so many doublets in the first place.
-        for (ToolChain * const tc : ToolChainManager::toolchains()) {
-            ToolChain *&bestTc = uniqueToolchains[tc->targetAbi()][tc->language()];
-            if (!bestTc) {
-                bestTc = tc;
+        const QList<ToolchainBundle> bundles = ToolchainBundle::collectBundles(
+            ToolchainBundle::HandleMissing::CreateAndRegister);
+        for (const ToolchainBundle &bundle : bundles) {
+            auto &bestBundle
+                = uniqueToolchains[bundle.targetAbi()][bundle.factory()->languageCategory()];
+            if (!bestBundle) {
+                bestBundle = bundle;
                 continue;
             }
 
-            if (bestTc->priority() > tc->priority())
-                continue;
-            if (bestTc->priority() < tc->priority()) {
-                bestTc = tc;
-                continue;
-            }
-
-            const QString bestFilePath = bestTc->compilerCommand().toString();
-            const QString currentFilePath = tc->compilerCommand().toString();
-            if (bestFilePath.contains("icecc"))
-                continue;
-            if (currentFilePath.contains("icecc")) {
-                bestTc = tc;
-                continue;
-            }
-
-            if (bestFilePath.contains("ccache"))
-                continue;
-            if (currentFilePath.contains("ccache")) {
-                bestTc = tc;
-                continue;
-            }
-
-            if (bestFilePath.length() > currentFilePath.length())
-                bestTc = tc;
+            if (ToolchainManager::isBetterToolchain(bundle, *bestBundle))
+                bestBundle = bundle;
         }
 
-        static const auto isHostKit = [](const Kit *kit) {
-            const Abi hostAbi = Abi::hostAbi();
-            if (HostOsInfo::isMacHost() && hostAbi.architecture() == Abi::ArmArchitecture) {
-                const Abi x86Abi(Abi::X86Architecture,
-                                 hostAbi.os(),
-                                 hostAbi.osFlavor(),
-                                 hostAbi.binaryFormat(),
-                                 hostAbi.wordWidth());
-
-                return kitMatchesAbiList(kit, {hostAbi, x86Abi});
-            }
-
-            return kitMatchesAbiList(kit, {hostAbi});
-        };
-
-        static const auto deviceTypeForKit = [](const Kit *kit) {
-            if (isHostKit(kit))
-                return Constants::DESKTOP_DEVICE_TYPE;
-            const QList<ToolChain *> toolchains = ToolChainKitAspect::toolChains(kit);
-            for (const ToolChain * const tc : toolchains) {
-                const Abi tcAbi = tc->targetAbi();
-                switch (tcAbi.os()) {
-                case Abi::BareMetalOS:
-                    return BareMetal::Constants::BareMetalOsType;
-                case Abi::BsdOS:
-                case Abi::DarwinOS:
-                case Abi::UnixOS:
-                    return RemoteLinux::Constants::GenericLinuxOsType;
-                case Abi::LinuxOS:
-                    if (tcAbi.osFlavor() == Abi::AndroidLinuxFlavor)
-                        return Android::Constants::ANDROID_DEVICE_TYPE;
-                    return RemoteLinux::Constants::GenericLinuxOsType;
-                case Abi::QnxOS:
-                    return Qnx::Constants::QNX_QNX_OS_TYPE;
-                case Abi::VxWorks:
-                    return "VxWorks.Device.Type";
-                default:
-                    break;
-                }
-            }
-            return Constants::DESKTOP_DEVICE_TYPE;
-        };
-
         // Create temporary kits for all toolchains found.
         decltype(resultList) tempList;
         for (auto it = uniqueToolchains.cbegin(); it != uniqueToolchains.cend(); ++it) {
             auto kit = std::make_unique<Kit>();
             kit->setSdkProvided(false);
             kit->setAutoDetected(false); // TODO: Why false? What does autodetected mean here?
-            for (ToolChain * const tc : it.value())
-                ToolChainKitAspect::setToolChain(kit.get(), tc);
+            for (const auto &bundle : it.value())
+                ToolchainKitAspect::setBundle(kit.get(), *bundle);
             if (contains(resultList, [&kit](const std::unique_ptr<Kit> &existingKit) {
-                return ToolChainKitAspect::toolChains(kit.get())
-                         == ToolChainKitAspect::toolChains(existingKit.get());
+                return ToolchainKitAspect::toolChains(kit.get())
+                         == ToolchainKitAspect::toolChains(existingKit.get());
             })) {
                 continue;
             }
@@ -354,7 +286,7 @@ void KitManager::restoreKits()
                 kit->setUnexpandedDisplayName(Tr::tr("Desktop (%1)").arg(it.key().toString()));
             else
                 kit->setUnexpandedDisplayName(it.key().toString());
-            DeviceTypeKitAspect::setDeviceTypeId(kit.get(), deviceTypeForKit(kit.get()));
+            RunDeviceTypeKitAspect::setDeviceTypeId(kit.get(), deviceTypeForKit(kit.get()));
             kit->setup();
             tempList.emplace_back(std::move(kit));
         }
@@ -371,7 +303,7 @@ void KitManager::restoreKits()
         });
         if (!abisOfBinary.isEmpty()) {
             for (auto it = tempList.begin(); it != tempList.end(); ++it) {
-                if (kitMatchesAbiOfBinary(it->get())) {
+                if (kitMatchesAbiList(it->get(), abisOfBinary)) {
                     kitForBinary = it->get();
                     resultList.emplace_back(std::move(*it));
                     tempList.erase(it);
@@ -379,7 +311,6 @@ void KitManager::restoreKits()
                 }
             }
         }
-        QList<Kit *> hostKits;
         if (!kitForBinary && !tempList.empty()) {
             const int maxWeight = tempList.front()->weight();
             for (auto it = tempList.begin(); it != tempList.end(); it = tempList.erase(it)) {
@@ -409,15 +340,20 @@ void KitManager::restoreKits()
     Kit *k = kitForBinary;
     if (!k)
         k = Utils::findOrDefault(resultList, Utils::equal(&Kit::id, defaultUserKit));
+    if (!k)
+        k = Utils::findOrDefault(hostKits, &Kit::isValid);
     if (!k)
         k = Utils::findOrDefault(resultList, &Kit::isValid);
     std::swap(resultList, d->m_kitList);
+    d->m_initialized = true;
     setDefaultKit(k);
 
     d->m_writer = std::make_unique<PersistentSettingsWriter>(settingsFileName(), "QtCreatorProfiles");
-    d->m_initialized = true;
-    emit m_instance->kitsLoaded();
-    emit m_instance->kitsChanged();
+
+    KitAspectFactory::handleKitsLoaded();
+
+    emit instance()->kitsLoaded();
+    emit instance()->kitsChanged();
 }
 
 KitManager::~KitManager()
@@ -430,20 +366,20 @@ void KitManager::saveKits()
     if (!d->m_writer) // ignore save requests while we are not initialized.
         return;
 
-    QVariantMap data;
-    data.insert(QLatin1String(KIT_FILE_VERSION_KEY), 1);
+    Store data;
+    data.insert(KIT_FILE_VERSION_KEY, 1);
 
     int count = 0;
     const QList<Kit *> kits = KitManager::kits();
     for (Kit *k : kits) {
-        QVariantMap tmp = k->toMap();
+        Store tmp = k->toMap();
         if (tmp.isEmpty())
             continue;
-        data.insert(QString::fromLatin1(KIT_DATA_KEY) + QString::number(count), tmp);
+        data.insert(numberedKey(KIT_DATA_KEY, count), variantFromStore(tmp));
         ++count;
     }
-    data.insert(QLatin1String(KIT_COUNT_KEY), count);
-    data.insert(QLatin1String(KIT_DEFAULT_KEY),
+    data.insert(KIT_COUNT_KEY, count);
+    data.insert(KIT_DEFAULT_KEY,
                 d->m_defaultKit ? QString::fromLatin1(d->m_defaultKit->id().name()) : QString());
     data.insert(KIT_IRRELEVANT_ASPECTS_KEY,
                 transform<QVariantList>(d->m_irrelevantAspects, &Id::toSetting));
@@ -455,52 +391,60 @@ bool KitManager::isLoaded()
     return d->m_initialized;
 }
 
-void KitManager::registerKitAspect(KitAspect *ki)
+bool KitManager::waitForLoaded(const int timeout)
 {
-    instance();
-    QTC_ASSERT(d, return);
-    d->addKitAspect(ki);
-
-    // Adding this aspect to possibly already existing kits is currently not
-    // needed here as kits are only created after all aspects are created
-    // in *Plugin::initialize().
-    // Make sure we notice when this assumption breaks:
-    QTC_CHECK(d->m_kitList.empty());
+    if (isLoaded())
+        return true;
+    showLoadingProgress();
+    QElapsedTimer timer;
+    timer.start();
+    while (!isLoaded() && !timer.hasExpired(timeout))
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+    return KitManager::isLoaded();
 }
 
-void KitManager::deregisterKitAspect(KitAspect *ki)
+void KitManager::showLoadingProgress()
 {
-    // Happens regularly for the aspects from the ProjectExplorerPlugin as these
-    // are destroyed after the manual call to KitManager::destroy() there, but as
-    // this here is just for sanity reasons that the KitManager does not access
-    // a destroyed aspect, a destroyed KitManager is not a problem.
-    if (d)
-        d->removeKitAspect(ki);
+    if (isLoaded())
+        return;
+    static QFutureInterface<void> futureInterface;
+    if (futureInterface.isRunning())
+        return;
+    futureInterface.reportStarted();
+    using namespace std::chrono_literals;
+    Core::ProgressManager::addTimedTask(futureInterface.future(),
+                                        Tr::tr("Loading Kits"),
+                                        "LoadingKitsProgress",
+                                        5s);
+    connect(instance(), &KitManager::kitsLoaded, []() { futureInterface.reportFinished(); });
 }
 
 void KitManager::setBinaryForKit(const FilePath &binary)
 {
     QTC_ASSERT(d, return);
-    d->setBinaryForKit(binary);
+    d->m_binaryForKit = binary;
 }
 
-QList<Kit *> KitManager::sortKits(const QList<Kit *> &kits)
+const QList<Kit *> KitManager::sortedKits()
 {
+    QTC_ASSERT(KitManager::isLoaded(), return {});
     // This method was added to delay the sorting of kits as long as possible.
     // Since the displayName can contain variables it can be costly (e.g. involve
     // calling executables to find version information, etc.) to call that
     // method!
     // Avoid lots of potentially expensive calls to Kit::displayName():
-    QList<QPair<QString, Kit *>> sortList = Utils::transform(kits, [](Kit *k) {
-        return qMakePair(k->displayName(), k);
+    std::vector<QPair<QString, Kit *>> sortList =
+        Utils::transform(d->m_kitList, [](const std::unique_ptr<Kit> &k) {
+        return qMakePair(k->displayName(), k.get());
     });
     Utils::sort(sortList,
                 [](const QPair<QString, Kit *> &a, const QPair<QString, Kit *> &b) -> bool {
-                    if (a.first == b.first)
-                        return a.second < b.second;
-                    return a.first < b.first;
+                    const int nameResult = Utils::caseFriendlyCompare(a.first, b.first);
+                    if (nameResult != 0)
+                        return nameResult < 0;
+                    return a.second < b.second;
                 });
-    return Utils::transform(sortList, &QPair<QString, Kit *>::second);
+    return Utils::transform<QList>(sortList, &QPair<QString, Kit *>::second);
 }
 
 static KitList restoreKitsHelper(const FilePath &fileName)
@@ -516,22 +460,22 @@ static KitList restoreKitsHelper(const FilePath &fileName)
                  qPrintable(fileName.toUserOutput()));
         return result;
     }
-    QVariantMap data = reader.restoreValues();
+    Store data = reader.restoreValues();
 
     // Check version:
-    int version = data.value(QLatin1String(KIT_FILE_VERSION_KEY), 0).toInt();
+    int version = data.value(KIT_FILE_VERSION_KEY, 0).toInt();
     if (version < 1) {
         qWarning("Warning: Kit file version %d not supported, cannot restore kits!", version);
         return result;
     }
 
-    const int count = data.value(QLatin1String(KIT_COUNT_KEY), 0).toInt();
+    const int count = data.value(KIT_COUNT_KEY, 0).toInt();
     for (int i = 0; i < count; ++i) {
-        const QString key = QString::fromLatin1(KIT_DATA_KEY) + QString::number(i);
+        const Key key = numberedKey(KIT_DATA_KEY, i);
         if (!data.contains(key))
             break;
 
-        const QVariantMap stMap = data.value(key).toMap();
+        const Store stMap = storeFromVariant(data.value(key));
 
         auto k = std::make_unique<Kit>(stMap);
         if (k->id().isValid()) {
@@ -542,7 +486,7 @@ static KitList restoreKitsHelper(const FilePath &fileName)
                      i);
         }
     }
-    const Id id = Id::fromSetting(data.value(QLatin1String(KIT_DEFAULT_KEY)));
+    const Id id = Id::fromSetting(data.value(KIT_DEFAULT_KEY));
     if (!id.isValid())
         return result;
 
@@ -557,6 +501,7 @@ static KitList restoreKitsHelper(const FilePath &fileName)
 
 const QList<Kit *> KitManager::kits()
 {
+    QTC_ASSERT(KitManager::isLoaded(), return {});
     return Utils::toRawPointer<QList>(d->m_kitList);
 }
 
@@ -565,6 +510,7 @@ Kit *KitManager::kit(Id id)
     if (!id.isValid())
         return nullptr;
 
+    QTC_ASSERT(KitManager::isLoaded(), return {});
     return Utils::findOrDefault(d->m_kitList, Utils::equal(&Kit::id, id));
 }
 
@@ -575,12 +521,13 @@ Kit *KitManager::kit(const Kit::Predicate &predicate)
 
 Kit *KitManager::defaultKit()
 {
+    QTC_ASSERT(KitManager::isLoaded(), return {});
     return d->m_defaultKit;
 }
 
-const QList<KitAspect *> KitManager::kitAspects()
+const QList<KitAspectFactory *> KitManager::kitAspectFactories()
 {
-    return d->kitAspects();
+    return KitAspectFactory::kitAspectFactories();
 }
 
 const QSet<Id> KitManager::irrelevantAspects()
@@ -598,15 +545,17 @@ void KitManager::notifyAboutUpdate(Kit *k)
     if (!k || !isLoaded())
         return;
 
-    if (Utils::contains(d->m_kitList, k))
-        emit m_instance->kitUpdated(k);
-    else
-        emit m_instance->unmanagedKitUpdated(k);
+    if (Utils::contains(d->m_kitList, k)) {
+        emit instance()->kitUpdated(k);
+        emit instance()->kitsChanged();
+    } else {
+        emit instance()->unmanagedKitUpdated(k);
+    }
 }
 
 Kit *KitManager::registerKit(const std::function<void (Kit *)> &init, Utils::Id id)
 {
-    QTC_ASSERT(isLoaded(), return nullptr);
+    QTC_ASSERT(isLoaded(), return {});
 
     auto k = std::make_unique<Kit>(id);
     QTC_ASSERT(k->id().isValid(), return nullptr);
@@ -623,165 +572,71 @@ Kit *KitManager::registerKit(const std::function<void (Kit *)> &init, Utils::Id
     if (!d->m_defaultKit || (!d->m_defaultKit->isValid() && kptr->isValid()))
         setDefaultKit(kptr);
 
-    emit m_instance->kitAdded(kptr);
+    emit instance()->kitAdded(kptr);
+    emit instance()->kitsChanged();
     return kptr;
 }
 
 void KitManager::deregisterKit(Kit *k)
 {
-    if (!k || !Utils::contains(d->m_kitList, k))
-        return;
-    auto taken = Utils::take(d->m_kitList, k);
-    if (defaultKit() == k) {
-        Kit *newDefault = Utils::findOrDefault(kits(), [](Kit *k) { return k->isValid(); });
-        setDefaultKit(newDefault);
+    deregisterKits({k});
+}
+
+void KitManager::deregisterKits(const QList<Kit *> kitList)
+{
+    QTC_ASSERT(KitManager::isLoaded(), return);
+    std::vector<std::unique_ptr<Kit>> removed; // to keep them alive until the end of the function
+    bool defaultKitRemoved = false;
+    for (Kit *k : kitList) {
+        QTC_ASSERT(k, continue);
+        std::optional<std::unique_ptr<Kit>> taken = Utils::take(d->m_kitList, k);
+        QTC_ASSERT(taken, continue);
+        if (defaultKit() == k)
+            defaultKitRemoved = true;
+        removed.push_back(std::move(*taken));
     }
-    emit m_instance->kitRemoved(k);
+
+    if (defaultKitRemoved) {
+        d->m_defaultKit = Utils::findOrDefault(kits(), &Kit::isValid);
+        emit instance()->defaultkitChanged();
+    }
+
+    for (auto it = removed.cbegin(); it != removed.cend(); ++it)
+        emit instance()->kitRemoved(it->get());
+    emit instance()->kitsChanged();
+
+    // FIXME: TargetSetupPage potentially deregisters kits on destruction, after the final
+    //        ICore::saveSettingsRequested() was emitted.
+    saveKits();
 }
 
 void KitManager::setDefaultKit(Kit *k)
 {
+    QTC_ASSERT(KitManager::isLoaded(), return);
+
     if (defaultKit() == k)
         return;
     if (k && !Utils::contains(d->m_kitList, k))
         return;
     d->m_defaultKit = k;
-    emit m_instance->defaultkitChanged();
+    emit instance()->defaultkitChanged();
 }
 
 void KitManager::completeKit(Kit *k)
 {
     QTC_ASSERT(k, return);
     KitGuard g(k);
-    for (KitAspect *ki : d->kitAspects()) {
-        ki->upgrade(k);
-        if (!k->hasValue(ki->id()))
-            ki->setup(k);
+    for (KitAspectFactory *factory : kitAspectFactories()) {
+        if (!k->isAspectRelevant(factory->id()))
+            continue;
+        factory->upgrade(k);
+        if (!k->hasValue(factory->id()))
+            factory->setup(k);
         else
-            ki->fix(k);
+            factory->fix(k);
     }
 }
 
-// --------------------------------------------------------------------
-// KitAspect:
-// --------------------------------------------------------------------
-
-KitAspect::KitAspect()
-{
-    KitManager::registerKitAspect(this);
-}
-
-KitAspect::~KitAspect()
-{
-    KitManager::deregisterKitAspect(this);
-}
-
-int KitAspect::weight(const Kit *k) const
-{
-    return k->value(id()).isValid() ? 1 : 0;
-}
-
-void KitAspect::addToBuildEnvironment(const Kit *k, Environment &env) const
-{
-    Q_UNUSED(k)
-    Q_UNUSED(env)
-}
-
-void KitAspect::addToRunEnvironment(const Kit *k, Environment &env) const
-{
-    Q_UNUSED(k)
-    Q_UNUSED(env)
-}
-
-QList<OutputLineParser *> KitAspect::createOutputParsers(const Kit *k) const
-{
-    Q_UNUSED(k)
-    return {};
-}
-
-QString KitAspect::displayNamePostfix(const Kit *k) const
-{
-    Q_UNUSED(k)
-    return QString();
-}
-
-QSet<Id> KitAspect::supportedPlatforms(const Kit *k) const
-{
-    Q_UNUSED(k)
-    return QSet<Id>();
-}
-
-QSet<Id> KitAspect::availableFeatures(const Kit *k) const
-{
-    Q_UNUSED(k)
-    return QSet<Id>();
-}
-
-void KitAspect::addToMacroExpander(Kit *k, MacroExpander *expander) const
-{
-    Q_UNUSED(k)
-    Q_UNUSED(expander)
-}
-
-void KitAspect::notifyAboutUpdate(Kit *k)
-{
-    if (k)
-        k->kitUpdated();
-}
-
-KitAspectWidget::KitAspectWidget(Kit *kit, const KitAspect *ki)
-    : m_kit(kit), m_kitInformation(ki)
-{
-    const Id id = ki->id();
-    m_mutableAction = new QAction(Tr::tr("Mark as Mutable"));
-    m_mutableAction->setCheckable(true);
-    m_mutableAction->setChecked(m_kit->isMutable(id));
-    m_mutableAction->setEnabled(!m_kit->isSticky(id));
-    connect(m_mutableAction, &QAction::toggled, this, [this, id] {
-        m_kit->setMutable(id, m_mutableAction->isChecked());
-    });
-}
-
-KitAspectWidget::~KitAspectWidget()
-{
-    delete m_mutableAction;
-}
-
-void KitAspectWidget::addToLayoutWithLabel(Layouting::LayoutItem &parentItem, QWidget *parent)
-{
-    QTC_ASSERT(parent, return);
-    auto label = createSubWidget<QLabel>(m_kitInformation->displayName() + ':');
-    label->setToolTip(m_kitInformation->description());
-    connect(label, &QLabel::linkActivated, this, [this](const QString &link) {
-        emit labelLinkActivated(link);
-    });
-
-    parentItem.addItem(label);
-    addToLayout(parentItem);
-    parentItem.addItem(Layouting::br);
-}
-
-void KitAspectWidget::addMutableAction(QWidget *child)
-{
-    QTC_ASSERT(child, return);
-    child->addAction(m_mutableAction);
-    child->setContextMenuPolicy(Qt::ActionsContextMenu);
-}
-
-QWidget *KitAspectWidget::createManageButton(Id pageId)
-{
-    auto button = createSubWidget<QPushButton>(msgManage());
-    connect(button, &QPushButton::clicked, this, [pageId] {
-        Core::ICore::showOptionsDialog(pageId);
-    });
-    return button;
-}
-
-QString KitAspectWidget::msgManage()
-{
-    return Tr::tr("Manage...");
-}
-
 // --------------------------------------------------------------------
 // KitFeatureProvider:
 // --------------------------------------------------------------------
@@ -816,7 +671,7 @@ QString KitFeatureProvider::displayNameForPlatform(Id id) const
         QTC_CHECK(!dn.isEmpty());
         return dn;
     }
-    return QString();
+    return {};
 }
 
 } // namespace ProjectExplorer
